#include "mbed.h"
#include "eeprom_flash.h"
#include "stm32f0xx_hal_flash.h"

 
/*
 * Debug option
 */
#if 0
//Enable debug
#include <cstdio>
#define DBG(x, ...) std::printf("[eeprom: DBG]"x"\r\n", ##__VA_ARGS__); 
#define WARN(x, ...) std::printf("[eeprom: WARN]"x"\r\n", ##__VA_ARGS__); 
#define ERR(x, ...) std::printf("[eeprom: ERR]"x"\r\n", ##__VA_ARGS__); 
#else
//Disable debug
#define DBG(x, ...) 
#define WARN(x, ...)
#define ERR(x, ...) 
#endif


int FLASH_PageErase(int page_addr)
{
    FLASH_EraseInitTypeDef er = {
        .TypeErase = FLASH_TYPEERASE_PAGES,
        .PageAddress = page_addr,
        .NbPages = 1,
    };
    uint32_t fault_sec = 0;
    HAL_FLASH_Unlock();
    HAL_StatusTypeDef res = HAL_FLASHEx_Erase(&er, &fault_sec);
    HAL_FLASH_Lock();

    printf("FLASH_PageErase : %d\n\r", res);

    return res == HAL_OK ? 0 : -1;
}

/*
 * Must call this first to enable writing
 */
void enableEEPROMWriting() {
    HAL_StatusTypeDef status = HAL_FLASH_Unlock();
    FLASH_PageErase(EEPROM_START_ADDRESS); // required to re-write
    CLEAR_BIT(FLASH->CR, FLASH_CR_PER); // Bug fix: bit PER has been set in Flash_PageErase(), must clear it here
    printf("enableEEPROMWriting : \n\r");
}
 
void disableEEPROMWriting() {
    HAL_FLASH_Lock();
    printf("disableEEPROMWriting : \n\r");
}
 
/*
 * Writing functions
 * Must call enableEEPROMWriting() first
 */
HAL_StatusTypeDef writeEEPROMHalfWord(uint32_t address, uint16_t data) {
    HAL_StatusTypeDef status;
    address = address + EEPROM_START_ADDRESS;
    
    status = HAL_FLASH_Program(FLASH_TYPEPROGRAM_HALFWORD, address, data);
    
    printf("writeEEPROMHalfWord : %d\n\r", status);
    return status;
}
 
HAL_StatusTypeDef writeEEPROMWord(uint32_t address, uint32_t data) {
    HAL_StatusTypeDef status;
    address = address + EEPROM_START_ADDRESS;
    
    status = HAL_FLASH_Program(FLASH_TYPEPROGRAM_WORD, address, data);
    printf("writeEEPROMWord : %d\n\r", status);
    return status;
}
 
/*
 * Reading functions
 */
uint16_t readEEPROMHalfWord(uint32_t address) {
    uint16_t val = 0;
    address = address + EEPROM_START_ADDRESS;
    val = *(volatile uint16_t*)address;
    printf("readEEPROMHalfWord : %d\n\r", val);
    return val;
}
 
uint32_t readEEPROMWord(uint32_t address) {
    uint32_t val = 0;
    address = address + EEPROM_START_ADDRESS;
    //val = *(__IO uint32_t*)address;
    val = *(volatile uint16_t*)address;
    printf("readEEPROMWord : %d\n\r", val);
    return val;
}